bl_info = {
    "name": "Link Scale",
    "author": "Joel Wagner, with help from batFINGER",
    "version": (0, 1),
    "blender": (2, 69, 0),
    "location": "3D View -> Properties -> Link Scale",
    "description": "Change object dimensions and keep them in proportion",
    "category": "Object"}


import bpy
from bpy.props import *
 
#
#    Store properties in the active scene
#
def link_scale(self, context):
    if self.link_object_scale:
        bpy.ops.object.link_scale('INVOKE_DEFAULT')


def initSceneProperties():
    bpy.types.Scene.link_object_scale = BoolProperty(
        name = "Link Scale",
        default = False,
        update = link_scale,
        description = "Change object dims proportionally")

 
initSceneProperties()
 
#
#    Menu in UI region
#
class UIPanel(bpy.types.Panel):
    bl_label = "Link Scale"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
     
    def draw_header(self, context):
        scn = context.scene
        self.layout.prop(scn, 'link_object_scale', text="")        
    
    def draw(self, context):
        layout = self.layout


        obj = context.object
        err = 0.0 in obj.scale
        if err:
           layout.label("ZERO Error", icon='ERROR')


        layout.label("Dimensions")
        row = layout.row(align=True)
        col = row.column()
        col.prop(context.object, "dimensions", text="")
        col = row.column()
        col.alignment = 'RIGHT'
        col.prop(context.object, "lock_scale", text="")




#
#   Actual script begins here
#


class LinkScale(bpy.types.Operator):
    """Link Scale when changing object dimensions"""
    bl_idname = "object.link_scale"
    bl_label = "Link Scale"
    _timer = None
    link_scale = None
    
    def modal(self, context, event):
        TOL = 0.00001
        
        if not context.scene.link_object_scale or event.type == 'ESC' or context.active_object is None:
            return self.cancel(context)
        
        obj_scale = self.obj_scale      
        if event.type == 'TIMER':
            obj = context.active_object
            if obj.name != self.object_name or not bpy.data.objects[self.object_name].select:
               return self.cancel(context)  # new object cancel.
               # or reset the object_name and obj_scale
               #self.object_name = obj.name
               #self.obj_scale = obj.scale.copy()
               
            if 0.0 in obj.scale:
                print("scale factor is zero")
                return self.cancel(context)
            
            if (obj.scale - obj_scale).length < TOL:
                return {'PASS_THROUGH'}
            
            if abs(obj.scale.x - obj_scale.x) > TOL and not obj.lock_scale[0]:
                scale_factor = obj.scale.x / obj_scale.x
                if not obj.lock_scale[1]:
                    obj.scale.y *= scale_factor
                if not obj.lock_scale[2]:    
                    obj.scale.z *= scale_factor
                
            elif abs(obj.scale.y - obj_scale.y) > TOL and not obj.lock_scale[1]:
                scale_factor = obj.scale.y / obj_scale.y
                if not obj.lock_scale[0]:                
                    obj.scale.x *= scale_factor
                if not obj.lock_scale[2]:                    
                    obj.scale.z *= scale_factor
                
            elif abs(obj.scale.z - obj_scale.z) > TOL and not obj.lock_scale[2]:
                scale_factor = obj.scale.z / obj_scale.z
                if not obj.lock_scale[0]:                
                    obj.scale.x *= scale_factor
                if not obj.lock_scale[1]:                    
                    obj.scale.y *= scale_factor
            
            self.obj_scale = obj.scale.copy()                                
          
               
        return {'PASS_THROUGH'}




    def invoke(self, context, event):
        if context.object is not None:
            obj_scale = context.active_object.scale.copy()
            self.obj_scale = obj_scale
            object_name = context.active_object.name
            self.object_name = object_name
            wm = context.window_manager
            self._timer = wm.event_timer_add(0.05, context.window)
            wm.modal_handler_add(self)
            return {'RUNNING_MODAL'}
        else:
            self.report({'WARNING'}, "No active object, could not finish")
            return {'CANCELLED'}




    def cancel(self, context):

        context.scene.link_object_scale = False
        wm = context.window_manager
        wm.event_timer_remove(self._timer)
        return {'CANCELLED'}        
    
#    Registration
def register():  
    bpy.utils.register_class(UIPanel)  
    bpy.utils.register_class(LinkScale)  



def unregister():
    bpy.utils.unregister_class(LinkScale)
    bpy.utils.unregister_class(UIPanel)
      
if __name__ == "__main__": 
    bpy.context.scene.link_object_scale = False 
    register()